#!/bin/sh
# selectphrases05.sh
# D. Gibbon
# Extract phoneme set and diphone set from text

# Version 05:
# - Added /Nw/ as phoneme

# Version 05:
# - Skips version 04 (unique unit count variant algorithm)
# - Uses pipes as additional boundary marker

# Version 03:
# - bug with "sort -rnu" corrected by separating into "sort -u | sort -rn"
# - units sorted rarest first, sentences sorted longest first
# - number of phonemes and number of units in selected sentences verified
#   with respect to numbers in whole corpus.

# CHECK FOR LONG VOWELS!! (done)
# Check for other digraphs
# Long EE is missing in corpus

#=======================================================================

# Phonetised text format:
# >-U-n-h-O-N-n-E-#-b-a-t-A-y-A-#-k-I-#-p-U-l-I-s-#-n-E-#-w-a-j-i-r-#-b-A-g-#-s-th-I-t-#-U-n-k-E-#-A-w-A-s-#-k-i-#-s-U-b-a-h-#-gh-E-r-A-b-a-n-d-i-k-a-r-#-l-i-#-o-r-#-U-n-h-En-#-n-a-m-A-j-#-a-t-A-#-k-a-r-n-E-#-k-E-#-l-I-E-#-b-A-h-a-r-#-n-a-h-i-n-#-j-A-n-E-#-d-I-y-A-#-g-a-y-A-<
# >-U-n-h-O-N-n-E-#-k-a-h-A-#-E-p-i-E-c-s-i-#-n-E-#-k-I-s-i-#-h-a-R-t-A-l-#-k-A-#-A-h-w-A-n-#-n-a-h-in-#-k-I-y-A-#-h-e-#-o-r-#-m-U-jh-E-#-s-a-m-a-jh-#-m-En-#-n-a-h-i-#-A-t-A-#-y-a-h-#-k-a-d-a-m-#-k-y-O-En-#-U-Th-A-y-A-#-g-a-y-A-#-h-e-<

# Unit format
#	#-w 758
#	#-y 252
#	>-A 32
#	>-D 2
# ...
#	>-w 86
#	>-y 30
#	A-# 2396
#	A-< 191
#	A-A 2

#=======================================================================

# Unit selection algorithm

# The idea is to select the minimum number of occurrences of units
# in the minimum number of phrases.

# The heuristic for finding the minimum number of phrases is to sort
# the corpus by sentence length on the assumption that the longer the
# phrase, the more units occur in the same phrase.

# The procedure used for finding the minimum number of phrases containing all
# units (assuming equal weighting of units) is
# - create a phonemic representation of the corpus
# - make a working copy of the phonemic representation of the corpus
# - sort the corpus copy in order of longest phrase first
#	[ the better criterion of phrases with most unique unit occurrences
#	first is NOT IMPLEMENTED ]
# - create a list of units (e.g. diphone units):
#       - make a list of all phoneme occurrences in order of occurrence
#       - make a copy of the list
#       - remove the last phoneme in the list and the first phoneme in the copy
#       - paste the list and the copy to make two columns (in fact a list
#               of diphone units)
#       - sort the units, adding unit counts and removing multiple occurrences
#       - sort in ascending order (i.e. rarest first), on the assumption that
#               if a rare unit is found in as long a phrase as possible, then
#               many other units will also have been found
#       - remove unit counts (in other procedures these may be needed)
# - create an initially empty set of selected phrases
# - for each unit, repeat:
#       - find the phrase with the first occurrence of the unit
#       - add this phrase to the set of selected phrases if not there already
#       - remove this phrase from the corpus copy [ NOT IMPLEMENTED ]
#       - add the set of selected phrases to the head of the corpus copy
# emit the final set of selected phrases

#=======================================================================


FILE=`echo $1 | sed "s/.txt$//g" | sed "s/.data$//g"`
CORPUS=$FILE.data
INFILE=$FILE-in.txt

STATS=$FILE-stats.txt

PHONETIZED=$FILE-phonetised.txt
SYLLABIFIED=$FILE-syllabified.txt
PHONEMES=$FILE-phonemes.txt
UNITSWEIGHTED=$FILE-unitsweighted.txt
UNITS=$FILE-units.txt
SELECTED=$FILE-selected.txt
READABLE=$FILE-readable.txt
SELCOUNTFILE=$FILE-selcount.csv
NUMBERED=$FILE-numbered.txt
SELECTEDWORDLIST=$FILE-selected-wordlist.txt

if [ ! -f $CORPUS ]
	then
		echo No input file. Input file has extension \".data\"
		exit
fi

#=======================================================================

echo Creating input file with separate phrases and normalising ...

cat $CORPUS |
tr "\015" "\012" |
tr "[-]" "\012" |
tr "[,]" "\012" |
tr "[.]" "\012" |
tr "[;]" "\012" |
tr "[:]" "\012" |
tr "[?]" "\012" |
tr "[!]" "\012" |
tr "[|]" "\012" |
grep . |
sed "s/  */ /g" |
sed "s/[-,.;:<>)(/\"\'\`]//g" |
tr "[" " " |
tr "]" " " |
sed "s/^  *//g" |
sed "s/  *$//g" |
sed "s/  */ /g" |
tr " " "#" |
# sed "s/#>/>/g" |
# sed "s/<#/</g" |
grep . > $INFILE

#=======================================================================

# Corpus format normalisation

echo Creating Parthas phonetized text format and for digraphs ...

# In some cases a digraph check will not be sufficient.
# Added Nw as phoneme 2004-06-22. DG
# Added syllabic nasals as mx, nx, Nx

cat $INFILE |
sed "
	s/^/>/g
	s/$/</g
	s/./&-/g
	s/^-*$//g
	s/-*$//g
	s/[>#]-m-m/#-mx-\\\$-m/g
	s/[>#]-n-n/#-nx-\\\$-n/g
	s/[>#]-N-N/#-Nx-\\\$-N/g
	s/N-w/Nw/g
	s/k-p/kp/g
	s/p-p/pp/g
	s/f-f/ff/g 
	s/t-t/tt/g
	s/k-k/kk/g
	s/d-d/dd/g 
	s/s-s/ss/g
	s/r-r/rr/g 
	s/m-m/mm/g
	s/n-n/nn/g
	s/N-N/NN/g
	s/a-a/aa/g
	s/i-i/ii/g
	s/e-e/ee/g
	s/u-u/uu/g
	s/o-o/oo/g
	s/I-I/II/g
	s/O-O/OO/g
	" > $PHONETIZED

echo Syllabification...

cat $PHONETIZED |
sed "
# Syllable initial only:
	s/-Nw/-\\\$&/g
	s/-kp/-\\\$&/g
	s/-f/-\\\$&/g
	s/-s/-\\\$&/g
	s/-h/-\\\$&/g
	s/-j/-\\\$&/g
	s/-w/-\\\$&/g
	s/-r/-\\\$&/g
	s/-R/-\\\$&/g
# Syllable final only (none)
# Word medial:
	s/\([ptkmnNbd]\)\(-[ptkbdmnNfhjJrRsw]\)/\1-\\\$\2/g
# Between Geminates (this one now looks redundant to me - DG):
	s/\([mnNptk]\)\([mnNptk]\)/\1-\\\$-\2/g
# Before intervocalic consonants:
	s/\([iIeaoOuV@\}]\)\(-[ptkbdmnNfhjJrRsw]-[iIeaoOuV@\}]\)/\1-\\\$\2/g
# V $ C:
# Remove $ after > (sentence boundary) and # (word boundary) and
# before < (sentence boundary) and # (word boundary):
	s/\([>#]\)-\$-/\1-/g
	s/-\$-\([<#]\)/-\1/g
" > $SYLLABIFIED

#=======================================================================
                                                                                
# Create list of phonemes in text for checking purposes

echo Creating list of phonemes for checking purposes ...
                                                                                
cat $PHONETIZED|
tr "-" "\012" |
tee tmp.phon |
sort -u |
grep . > $PHONEMES

####################################

#=======================================================================

# Create list of diphone units

# tmp.phon contains text with phoneme occurrences listed one per line
# Version 03: "rarest first ordering"

echo Creating list of diphone units...

cat tmp.phon |
tail +2 > tmp.second
paste tmp.phon tmp.second |
tr "\011" "-" |
grep -v "<->" |
grep -v "\-$" |
sort |
uniq -c |
tr "\011" " " |
sed "s/^ *//g" |
sed "s/  */ /g" |
grep . |
sort -n | 
gawk '
	{print $2, $1}
' |
tee $UNITSWEIGHTED |
gawk '
	{print $1}
' > $UNITS

#=======================================================================

# Measure length of phrases (replaced by unique unit count)

# Heuristic sentence length measure (quite fast)
echo Measuring phrase length...

cat $PHONETIZED |
gawk -F"-" '{print NF,$0}' |
sort -u |
sort -nr |
grep .  > tmp.oldinput
cat tmp.oldinput |
sed "s/[<>-]//g" |
sed "s/#/ /g" > $NUMBERED

## Unique unit per phrase count (very slow!!!)
#
#rm -f tmp.input
#touch tmp.input
#
#echo Ordering phrases by unique unit count ... \(may take several minutes\)
#
#for STRING in `cat $PHONETIZED`
#do
#
#	echo $STRING |
#	tr "-" "\012" |
#	tee tmp.phon |
#	sort -u |
#	grep . > tmp.1
#        cat tmp.1 |
#        tail +2 > tmp.2
#
#	paste tmp.1 tmp.2 |
#	grep -v "\-$" |
#	sort -u > tmp.3
#	echo `wc -l tmp.3 | gawk '{print $1}'` $STRING >> tmp.input
#	wc -l tmp.input | gawk '{print $1}'
#done
#
#cat tmp.input |
#sort -nr > tmp.oldinput

#=======================================================================

# For each unit, select first phrase in which it occurs

echo Unit selection ... \(may take some time\)

rm -f $SELECTED
touch $SELECTED

SELCOUNT=""
rm -f $SELCOUNTFILE
touch $SELCOUNTFILE

for UNIT in `cat $UNITS`
do
	SELCOUNT=`wc -l $SELECTED | gawk '{print $1}'`
#	echo " " $SELCOUNT selections at unit $UNIT
	echo $SELCOUNT >> $SELCOUNTFILE
	grep $UNIT tmp.oldinput |
	head -1 |
	grep . >> $SELECTED
# Use numeric sort:
        sort -u $SELECTED |
        sort -nr > tmp.selected
        mv -f tmp.selected $SELECTED
# Use this order to ensure prepending:
        cat $SELECTED tmp.oldinput > tmp.newinput
        mv -f tmp.newinput tmp.oldinput
done 

#=======================================================================

# Output selection progress CSV file

echo Storing selection progress count...

cat $SELCOUNTFILE |
gawk '{print $0, NF}' |
sort -n |
uniq -c |
gawk '{print $2 "\t" $1}' |
sort -n > tmp
mv -f tmp $SELCOUNTFILE

#=======================================================================

echo Emitting selected phrases in Parthas format and in readable format ...

rm -f $READABLE
touch $READABLE

echo Selected Ibibio sentences \(`date`\) >> $READABLE
echo `whoami`, `date`, $0 >> $READABLE
echo "" >> $READABLE

cat $SELECTED | gawk '{print $2}' |
tee tmp.selected |
sed "s/[-><]//g" |
sed "s/#/ /g" |
tee tmp.wordlist |
gawk '
	{
	print NR ": " $0
	print "" 
	}
' >> $READABLE

mv -f tmp.selected $SELECTED

#=======================================================================

echo Creating selected wordlist ...

cat tmp.wordlist |
tr " " "\012" |
grep . |
sort |
uniq -c |
sed "s/ //g" |
tr "\011" " " |
sort -rn |
gawk '{print $2}' > $SELECTEDWORDLIST

#=======================================================================
#=======================================================================

# Create list of phonemes in selected text for checking purposes

# rm -f tmp*

echo Verifying selected wrt original units:

# Create list of phonemes in text for checking purposes
echo " Creating list of phonemes for checking ..."

cat $SELECTED|
tr "-" "\012" |
tee tmp.phon |
sort -u |
grep . > $PHONEMES.selected

#=======================================================================

# Create list of diphone units

# tmp.phon contains text with phoneme occurrences listed one per line
# Version 03: "rarest first ordering"

echo " Creating list of diphone units..."

cat tmp.phon |
tail +2 > tmp.second
paste tmp.phon tmp.second |
tr "\011" "-" |
grep -v "<->" |
grep -v "\-$" |
sort |
uniq -c |
tr "\011" " " |
sed "s/^ *//g" |
sed "s/  */ /g" |
grep . |
sort -n |
gawk '
        {print $2, $1}
' |
tee $UNITSWEIGHTED.selected|
gawk '
        {print $1}
' > $UNITS.selected

#=======================================================================

echo " Comparing unit lists..."

sort $UNITS > tmp.a
sort $UNITS > tmp.b
echo Verification: `comm -3 tmp.a tmp.b | wc -l` \(0 means identity, i.e. ok\)

#=======================================================================

rm -f $STATS

PHONCOUNT=`wc -l $PHONEMES | gawk '{print $1}'`
UNITCOUNT=`wc -l $UNITS | gawk '{print $1}'`
SELCOUNT=`wc -l $SELECTED | gawk '{print $1}'`
CORPCOUNT=`wc -l $PHONETIZED | gawk '{print $1}'`

RATIO=`gawk 'BEGIN {print 100*ARGV[1]/ARGV[2] "%"}' $SELCOUNT $CORPCOUNT`

echo =================================================== > $STATS
echo Item counts: >> $STATS
echo " Phoneme count: " $PHONCOUNT >> $STATS
echo " Unit count:    " $UNITCOUNT >> $STATS
echo " Selection size:" $SELCOUNT >> $STATS
echo " Corpus size:   " $CORPCOUNT >> $STATS
echo " Size ratio:    " $RATIO >> $STATS
echo =================================================== >> $STATS

cat $STATS

#=======================================================================

rm -f tmp.*

echo Done.

#=======================================================================

