#! /bin/sh
# ibibio_fst_08.sh
# D. Gibbon
#================================================================
# History:

# ibibio_fst_01.sh
# 2004-02-03
# 2004-02-06
# Plain FST with no cooccurrence restrictions.

# ibibio_fst_02.sh
# 2004-02-05
# 2004-02-09
# Enhanced FST with registers for cooccurrence restrictions.

# ibibio_fst_03.sh
# 2004-02-09
# Substitution of Tense yaa for maa corrected (Bruce Connell).
# History and description added.
# Registers given functional names: subject, object
# Proper names added at subject and object positions

# ibibio_fst_04.sh
# 2004-02-09
# Simplified generator code by removal of general conditions

# ibibio_fst_05.sh
# 2004-02-10
# Separate tonal tier

# ibibio_fst_06.sh
# 2004-02-16
# Further corrections from Bruce Connell

# ibibio_fst_07.sh
# 2004-02-19
# Further corrections from Eno Urua
# Added object agreement prefix
# NPs except pronouns commented out

# ibibio_fst_08.sh
# 2004-02-22
# Added grammatical tone tape
# NPs except pronouns commented out
# Corrections: AGR2 2nd person object verbal prefix

# ibibio_fst_09.sh
# 2004-02-22
# Change: distinguish between
# - AGR_1 as auxiliary prefix, AGR_2 as main prefix
# - AGR_1 only has subj pref, and only occurs if there is an aux,
# - AGR_2 has obj pref if 2nd person object
# Previously the other way round.

#================================================================
# Description:

# 1. Nondeterministic FST implementation in gawk.
# 2. The gawk script is enclosed in a shell script wrapper.
# 3. Non-deterministic processing is implemented by sequences of
#    function calls with the same input category, but outputs based
#    on specific subcategories (values). 
# 4. Concord constraints are expressed using register variables in order to
#    avoid explosively multiplying the number of transitions:
#    	subnumber, subperson, tense (later: other categories, e.g. negation)
# 5. Registers are used in order to generalise over dependencies (possibly
#    discontinuous) and avoid multiplying sub-paths in the FST.
# 6. The morphosyntactic model is non-cyclic.

#================================================================

# Input checks

# INPUTSTRING=$1
INPUTSTRING="subject tense verb object"
#================================================================

# gawk script

gawk '

# I/O

BEGIN {
	counter=0;
	q_subject("","","");
	}

#================================================================

# FST transitions

#================================================================
# SUBJECT NODE: set SUBJECT registers

function q_subject(out, tones, gramtones, lextones, cats){

# Epsilon transition:
# Cut out in order to get pronouns only
#	q_subjquant(out, tones, gramtones, lextones, cats " [SUBJECT:");
#
# Proper name:
#	subnumber="sing"; subperson="third";
#	q_concord(out " [Ime]", tones " [L H]", gramtones " [ ]", lextones " [L H]", cats " [SUBJECT: [propname: " subnumber " " subperson " =Ime]]");

# Personal pronouns:

# First:
	subperson="first";
	subnumber="sing";
	q_concord(out " [ami]", tones " [L L]", gramtones " [L L]", lextones " [ ]", cats " [SUBJECT: [pron: " subnumber " " subperson "=I]]");
	subnumber="plur";
	q_concord(out " [nnjIn]", tones " [L L]", gramtones " [L L]", lextones " [ ]", cats " [SUBJECT: [pron: " subnumber " " subperson " =we]]");

# Second:
	subperson="second";
	subnumber="sing";
	q_concord(out " [afo]", tones " [L L]", gramtones " [L L]", lextones " [ ]", cats " [SUBJECT: [pron: " subnumber " " subperson "=you]]");
	subnumber="plur";
	q_concord(out " [ndufo]", tones " [L L L]", gramtones " [L L]", lextones " [ ]", cats " [SUBJECT: [pron: " subnumber " " subperson " =you]]");

# Third:
	subperson="third";
	subnumber="sing";
	q_concord(out " [anje]", tones " [L H]", gramtones " [L H]", lextones " [ ]", cats " [SUBJECT: [pron: " subnumber " " subperson "=he-she-it]]");
	subnumber="plur";
	q_concord(out " [OmmO]", tones " [LH HL]", gramtones " [LH HL]", lextones " [ ]" , cats " [SUBJECT: [pron: " subnumber " " subperson " =they]]");
	}

#================================================================
# SUBJECT QUANTIFIER NODE

function q_subjquant(out, tones, gramtones, lextones, cats){
	subquant="yes";
	subperson="third";
	subnumber="sing";
	q_subjadj(out, tones, gramtones, lextones, cats);

	subnumber="plur";
	q_subjnounnum(out " [uwak]", tones " [L H]", gramtones " [L H]", lextones " [ ]", cats " [quant: =many]");
	q_subjnounnum(out " [umiaN]", tones " [L H]", gramtones " [L H]", lextones " [ ]", cats " [quant: =many]");
	q_subjnounnum(out " [usVk]", tones " [L H]", gramtones " [L H]", lextones " [ ]", cats " [quant: =some]");
	q_subjnounnum(out " [ubaak]", tones " [H D]", gramtones " [H D]", lextones " [ ]", cats " [quant: =some]");
	q_subjnounnum(out " [afIt]", tones " [L H]", gramtones " [L H]", lextones " [ ]", cats " [quant: =all]");
	q_subjnounnum(out " [ukeet]", tones " [H L]", gramtones " [H L]", lextones " [ ]", cats " [quant: =all]");
	}

#================================================================
# SUBJECT NUMBER NODE

# Note mme -> mm /_V postlexically

function q_subjnounnum(out, tones, gramtones, lextones, cats){
        if (subnumber=="plur")
                q_subjadj(out " [mme]", tones " [L L]", gramtones " [L L]", lextones " [ ]", cats " [agr: " subnumber " " subperson);
        else
                q_subjadj(out, tones, gramtones, lextones, cats " [agr: " subnumber " " subperson "]");
        }

#================================================================
# SUBJECT ADJECTIVE NODE (Urua:87)

function q_subjadj(out, tones, gramtones, lextones, cats){
	q_subjnoun(out,tones,lextones, cats);
	q_subjnoun(out " [afja]", tones " [L H]", gramtones " [ ]", lextones " [L H]", cats " [adj: =white]");
	}

#================================================================
# SUBJECT NOUN NODE

function q_subjnoun(out, tones, gramtones, lextones, cats){
	if (subquant=="yes")
		tone_1 = "H(L)";
	else
		 tone_1 = "H";
# No, term of respect for older woman (Connell)
#	q_subjdet(out " [nne]", tones " [" tone_1 " L]", gramtones " [ ]", lextones " [ ]", cats " =girl]");
	q_subjdet(out " [edON]", tones " [H HL]", gramtones " [ ]", lextones " [H HL]", "[noun: " cats " =sheep]");
	}

#================================================================
# SUBJECT DETERMINER NODE

function q_subjdet(out, tones, gramtones, lextones, cats){
#	if (subquant=="yes"){
		q_concord(out, tones, gramtones, lextones, cats "]");
#		}
#	else {
		q_concord(out " [emi]", tones " [L L]", gramtones " [L L]", lextones " [ ]", cats " [det: prox =this]]");
		q_concord(out " [odo]", tones " [L L]", gramtones " [L L]", lextones " [ ]", cats " [det: dist =that]]");
#		}
	}

#================================================================
# AGR_2 OBJECT NODE

# Underlyingly SOV?

function q_concord(out, tones, gramtones, lextones, cats){
	obperson="first";
	obnumber="sing";
	q_subjectagr(out, tones, gramtones, lextones, cats);
	obnumber="plur";
	q_subjectagr(out, tones, gramtones, lextones, cats);

	obperson="second";
	obnumber="sing";
	q_subjectagr(out, tones, gramtones, lextones, cats);
	obnumber="plur";
	q_subjectagr(out, tones, gramtones, lextones, cats);

	obperson="third";
	obnumber="sing";
	q_subjectagr(out, tones, gramtones, lextones, cats);
	obnumber="plur";
	q_subjectagr(out, tones, gramtones, lextones, cats);
	}

#================================================================
# AGR SWITCH NODE FOR TENSE - OBJECT CONCORD

function q_subjectagr(out, tones, gramtones, lextones, cats){

	tensecat="present";

	if (obperson=="second")
		q_verbprefix(out, tones, gramtones, lextones, cats);
	else q_agr(out, tones, gramtones, lextones, cats);

	tensecat="nonpresent";
	q_agr(out, tones, gramtones, lextones, cats);

	}

#================================================================
# AGR_1 SUBJECT CONCORD

function q_agr(out, tones, gramtones, lextones, cats){

	cats=" [verb:";
	lextones=" [ ]";

	if (subperson=="first"){
		if (subnumber=="sing"){
			out=out " [n-]"; tones=tones " [H]"; gramtones "[H]";
			q_tense(out, tones, gramtones, lextones, cats);
			}
		else {
			out=out " [i-]"; tones=tones " [L]"; gramtones "[L]";
			q_tense(out, tones, gramtones, lextones, cats);
			}
		}
	if (subperson=="second"){
		if (subnumber=="sing"){
			out=out " [a-]"; tones=tones " [L]"; gramtones "[L]";
			q_tense(out, tones, gramtones, lextones, cats);
			}
		else {
			out=out " [e-]"; tones=tones " [L]"; gramtones "[L]";
			q_tense(out, tones, gramtones, lextones, cats);
			}
		}
	if (subperson=="third"){
		if (subnumber=="sing"){
			out=out " [a-]"; tones=tones " [H]"; gramtones "[H]";
			q_tense(out, tones, gramtones, lextones, cats);
			}
		else {
			out=out " [e-]"; tones=tones " [H]"; gramtones "[H]";
			q_tense(out, tones, gramtones, lextones, cats);
			}
		}

	}


#================================================================
# TENSE VERB PREFIX NODE: set TENSE inflection register

# proximal vs. distal tense (Connell)

function q_tense(out, tones, gramtones, lextones, cats){

	if (tensecat=="present"){
		q_verbprefix(out, tones, gramtones, lextones, cats);
		}

	else{
		tense=" [future]"; tensemorph=" [yaa-]";
		scale=" [prox]";  scalemorph=" [L H]"
		q_verbprefix(out tensemorph, tones scalemorph, gramtones scalemorph, lextones " [ ]", cats tense scale);
		scale=" [dist]";  scalemorph=" [H L]"
		q_verbprefix(out tensemorph, tones scalemorph, gramtones scalemorph, lextones " [ ]", cats tense scale);

		tense=" [past]"; tensemorph=" [maa-]";
		scale=" [prox]";  scalemorph=" [L H]"
		q_verbprefix(out tensemorph, tones scalemorph, gramtones scalemorph, lextones " [ ]", cats tense scale);
		scale=" [dist]";  scalemorph=" [H L]"
		q_verbprefix(out tensemorph, tones scalemorph, gramtones scalemorph, lextones " [ ]", cats tense scale);
		}
	}

#================================================================
# AGR_2 VERB PREFIX NODE:

function q_verbprefix(out, tones, gramtones, lextones, cats){

	objcats= cats " [objagr: " obperson obnumber "]";

	if (obperson=="second"){
		tones=tones " [ H]";
		gramtones=gramtones " [ H]";
		lextones=lextones " [ ]";
		if (obnumber=="sing") out=out " [u-]";
		else out=out " [i-]";
		}

	q_verb(out, tones, gramtones, lextones, objcats);
	}


#================================================================
# VERB NODE:

function q_verb(out, tones, gramtones, lextones, cats){
		q_object(out " [bed]", tones " [H]", gramtones " [ ]", lextones " [H]", cats " =await]");
#		q_object(out " [bed]", tones " [L]", gramtones " [ ]", lextones " [L]", cats " =push]");
	}


#================================================================
# OBJECT NODE: set OBJECT registers

function q_object(out, tones, gramtones, lextones, cats){

# Cut out in order to get pronouns only
# Epsilon transition:
#	q_objquant(out, tones, gramtones, lextones, cats " [OBJECT:");
#
# Proper name:
#	obnumber="sing"; obperson="third";
#	q_end(out " [Ime]", tones " [L H]", gramtones " [ ]", lextones " [L H]", cats " [OBJECT: [propname: " obnumber " " obperson " =Ime]]");

# Personal pronouns (NOTE initial H tone on 1st, 2nd):

# First:
	if(obperson=="first"){
		if(obnumber=="sing"){
		q_end(out " [miin]", tones " [H L]", gramtones "[H L]", lextones " [ ]", cats " [OBJECT: [pron: " obnumber " " obperson " =I]]");
#		q_end(out " [mien]", tones " [H L]", gramtones " [H L]", lextones " [ ]", cats " [OBJECT: [pron: " obnumber " " obperson " =I]]");
		}
		if (obnumber=="plur"){
		q_end(out " [nnjIn]", tones " [H L]", gramtones " [H L]", lextones " [ ]", cats " [OBJECT: [pron: " obnumber " " obperson " =we]]");
		}
	}

# Second:
        if(obperson=="second"){
        	if(obnumber=="sing"){
		q_end(out " [fiin]", tones " [H L]", gramtones " [H L]", lextones " [ ]", cats " [OBJECT: [pron: " obnumber " " obperson " =you]]");
#		q_end(out " [fien]", tones " [H L]", gramtones " [H L]", lextones " [ ]", cats " [OBJECT: [pron: " obnumber " " obperson " =you]]");
		}
        	if(obnumber=="plur"){
		q_end(out " [ndufo]", tones " [H L L]", gramtones " [H L L]", lextones " [ ]", cats " [OBJECT: [pron: " obnumber " " obperson " =you]]");
		}
	}

# Third:
        	if(obperson=="third"){
        	if(obnumber=="sing"){
		q_end(out " [anje]", tones " [L H]", gramtones " [L H]", lextones " [ ]", cats " [OBJECT: [pron: " obnumber " " obperson " =he-she-it]]");
		}
        	if(obnumber=="plur"){
		q_end(out " [OmmO]", tones " [LH HL]", gramtones " [L H]", lextones " [ ]", cats " [OBJECT: [pron: " obnumber " " obperson " =they]]");
		}
		}
	}

#================================================================
# OBJECT QUANTIFIER NODE

function q_objquant(out, tones, gramtones, lextones, cats){
	obquant="yes";
	obperson="third";
	obnumber="sing";
	q_objnounnum(out, tones, gramtones, lextones, cats);

	obnumber="plur";
	q_objnounnum(out " [uwak]", tones " [L H]", gramtones " [L H]", lextones " [ ]", cats " [quant: =many]");
	q_objnounnum(out " [umiaN]", tones " [L H]", gramtones " [L H]", lextones " [ ]", cats " [quant: =many]");
	q_objnounnum(out " [usVk]", tones " [L H]", gramtones " [L H]", lextones " [ ]", cats " [quant: =some]");
	q_objnounnum(out " [ubaak]", tones " [H D]", gramtones " [H D]", lextones " [ ]", cats " [quant: =some]");
	q_objnounnum(out " [afIt]", tones " [L H]", gramtones " [L H]", lextones " [ ]", cats " [quant: =all]");
	q_objnounnum(out " [ukeet]", tones " [H L]", gramtones " [H L]", lextones " [ ]", cats " [quant: =all]");
	}

#================================================================
# OBJECT NUMBER NODE
# Note that mme -> mm /_V

function q_objnounnum(out, tones, gramtones, lextones, cats){
        if (obnumber=="plur")
                q_objadj(out " [mme]", tones " [L L]", gramtones " [L L]", lextones " [ ]", cats " [agr: " subnumber " " subperson "]");
        else
                q_objadj(out, tones, gramtones, lextones, cats " [agr: " obnumber " " obperson "]");
        }

#================================================================
# OBJECT ADJECTIVE NODE (Urua:87)

function q_objadj(out, tones, gramtones, lextones, cats){
	q_objnoun(out, tones, lextones, cats);
	q_objnoun(out " [utu]", tones " [L L]", gramtones " [ ]", lextones " [L L]", cats " [adj: =yellow]");
	}

#================================================================
# OBJECT NOUN NODE

function q_objnoun(out, tones, gramtones, lextones, cats){
	q_objdet(out " [unen]", tones " [H HL]", gramtones "[ ]", lextones " [H HL]" , cats " [noun: " subnumber " " subperson " =chicken]");
	}

#================================================================
# OBJECT DETERMINER NODE

function q_objdet(out, tones, gramtones, lextones, cats){
	if (obquant!="yes"){
# Corrected from L L (Connell)
# Also q_end for erroneous q_concord
	q_end(out " [emi]", tones " [H L]", gramtones " [H L]", lextones " [ ]", cats " [det: prox =this]]");
	q_end(out " [odo]", tones " [L L]", gramtones " [L L]", lextones " [ ]", cats " [det: dist =that]]");
	} }

#================================================================
# PREP PHRASE - insert here !!!
# e.g. k/e /ob/ot = on the hill

#================================================================
# FINAL NODE:

function q_end(out, tones, gramtones, lextones, cats) {
#	print ++counter ":\nLEXTONE:" lextones "\n   TONE:" tones "\n   SEGM:" out "\n  MORPH: [s:" cats "]\n";
	print ++counter ":"
#	print " LEXTONE:" lextones;
#	print "GRAMTONE:" gramtones;
	print "    TONE:" tones;
	print "    SEGM:" out;
#	print "   MORPH: [s:" cats "]";
#	print "";
	}

#================================================================

' |

tr "[" "X" | tr "]" "X" |
sed "s/  */ /g" |
sed "s/X X/X+X/g" |
tr "X" " " |
sed "s/-//g"
#EOF=============================================================
